<?php
/**
 * Order export hydrator
 *
 * Copyright (C) 2021 Leipzig University Library
 *
 * PHP Version 7
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * @category VuFind
 * @package  Hydrator
 * @author   JP Kanter <kanter@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 * @link     https://vufind.org/wiki/development Wiki
 */
namespace fid\Hydrator;

use fid\Service\DataTransferObject\Order;
use Zend\Http\Request as HttpRequest;
use Zend\Hydrator\AbstractHydrator;
use Zend\Stdlib\RequestInterface as Request;

/**
 * Order hydrator for saving aquistions data correctly to dto before sending to fidis
 *
 * @category VuFind
 * @package  Hydrator
 * @author   JP Kanter <kanter@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 * @link     https://vufind.org/wiki/development Wiki
 */
class OrderExportHydrator extends AbstractHydrator
{
    public const DATE_FORMAT = 'Y-m-d H:i:s';
    public const SUBITO_ARTICLE = "subito-article";
    public const SUBITO_PARTIAL_COPY = "subito-partial-copy";
    public const DIGITIZATION = "digitization";

    public const FIELDS = ["id", "type", "createdAt", "orderedBy", "fullname", "deliveryAddress", "title",
        "responsible", "published", "titleLink", "status", "dueDate", "ownerLib", "signature",
        "externalUrl", "comment"];

    /**
     * Request object for return value
     *
     * @var Request
     */
    protected $request;

    /**
     * Extract method:
     * Compiles available order and user data
     *
     * @param Order $object Order object
     *
     * @return array
     */
    public function extract($object)
    {
        foreach (self::FIELDS as $defaultKey) {
            $bloated[$defaultKey] = '';
        }
        $bloated['id'] = $object->getId();
        $bloated['type'] = $object->getType();
        $bloated['label'] = $object->getLabel();
        $bloated['orderedBy'] = $object->getUser()->getUsername();
        $bloated['fullname'] = $object->getUser()->getFirstname() . " " . $object->getUser()->getLastname();
        $bloated['deliveryAddress'] = $object->getUser()->getDeliveryAddress();
        $bloated['createdAt'] = $object->getCreatedAt() ? $object->getCreatedAt()->format(self::DATE_FORMAT) : '';
        $bloated['dueDate'] = $object->getDueDate() ? $object->getDueDate()->format(self::DATE_FORMAT) : '';
        $bloated['status'] = $object->getStatus() ?? 'open';
        // extract data
        $data = $object->getData();
        $bloated['titleLink'] = $data['record']['url'] ?? '';
        if (isset($data['digitization']) and !empty($data['digitization'])) {
            $digi = $data['digitization'] ?? '';
            $bloated['comment'] = $digi['comment'] ?? '';
            $bloated['ownerLib'] = $digi['library'] ?? '';
            $bloated['externalUrl'] = $digi['external_url'] ?? '';
            $bloated['signature'] = $digi['signature'] ?? '';
            $bloated['title'] = $digi['title'] ?? $bloated['label'] ?? '';
            $bloated['responsible'] = $digi['responsible'] ?? '';
            $bloated['published'] = $digi['year'] ?? '';
        }
        if (isset($data['pda']) and !empty($data['pda'])) {
            $bloated['comment'] = $data['pda']['comment'] ?? '';
        }

        return $bloated;
    }

    /**
     * Not implemented
     *
     * @param array $data   Relevant data
     * @param Order $object Order object
     *
     * @return Order|void
     */
    public function hydrate(array $data, $object)
    {
    }

    /**
     * Helper function to convert object member variables to array.
     * Member name becomes array key.
     *
     * @param $object Object
     *
     * @return array
     */
    protected function toArray($object) : array
    {
        $array = (array)$object;
        $keys = str_replace('*', '', array_keys($array));
        return array_combine($keys, array_values($array));
    }

    /**
     * Functionality to ease export to csv-like files
     *
     * @param $object     Order  Object
     * @param $seperator  string seperator for comma based files
     * @param $filtersort array  orders/filters the fields in order of the array, whitespace for non-existing fields
     *
     * @return string
     */
    public function toCSV(Order $object, string $seperator="\t", array $filtersort=[]) : string
    {
        $arrayed = self::extract($object);
        $line = "";
        if (empty($filtersort)) {
            foreach ($arrayed as $field) {
                $line .= $field . $seperator;
            }
        }

        return rtrim($line, $seperator);
    }

    /**
     * Get request object
     *
     * @return Request
     */
    public function getRequest()
    {
        if (! $this->request) {
            $this->request = new HttpRequest();
        }

        return $this->request;
    }
}
