<?php
/**
 * User Export Hydrator
 *
 * Copyright (C) 2019 Leipzig University Library
 *
 * PHP Version 7
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * @category VuFind
 * @package  Hydrator
 * @author   JP Kanter <kanter@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 * @link     https://vufind.org/wiki/development Wiki
 */
namespace fid\Hydrator;

use fid\Service\DataTransferObject\User;
use Zend\Hydrator\AbstractHydrator;

/**
 * User Hydrator
 * Gets functionality by using UserDataStrategy
 *
 * @category VuFind
 * @package  Hydrator
 * @author   JP Kanter <kanter@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 * @link     https://vufind.org/wiki/development Wiki
 *
 * @see UserHydratorDelegatorFactory
 */
class UserExportHydrator extends AbstractHydrator
{
    public const DATE_FORMAT = 'Y-m-d H:i:s';
    public const FIELDS = ["id", "username", "academicTitle", "firstname", "lastname",
        "homeLibrary", "college", "jobTitle", "addresses", "permissions", "data", "orders"];
    public const FIND = ["<i>", "</i>", "\n", "\t"];
    public const REPLACE = ["", "", " ", "   "];

    /**
     * Extract method:
     * Compiles available order and user data
     *
     * @param User $object User object
     *
     * @return array
     */
    public function extract($object)
    {
        foreach (self::FIELDS as $defaultKey) {
            $bloated[$defaultKey] = '';
        }
        $bloated['id'] = $object->getId() ?? -1;
        $bloated['username'] = $object->getUsername() ?? '';
        $bloated['academicTitle'] = $object->getAcademicTitle() ?? '';
        $bloated['firstname'] = $object->getFirstname() ?? '';
        $bloated['lastname'] = $object->getLastname() ?? '';
        $bloated['homeLibrary'] = $object->getHomeLibrary() ?? '';
        $bloated['college'] = $object->getCollege() ?? '';
        $bloated['jobTitle'] = $object->getJobTitle() ?? '';
        $bloated['addresses'] = $object->getAddresses() ?? [];
        $bloated['permissions'] = $object->getPermissions() ?? [];
        $bloated['data'] = $object->getData() ?? [];
        $bloated['orders'] = $object->getOrders() ?? [];
        return $bloated;
    }

    /**
     * Not implemented
     *
     * @param array $data   Relevant data
     * @param User $object  Order object
     *
     * @return User|void
     */
    public function hydrate(array $data, $object)
    {
    }

    /**
     * Extracts data and then flattens array fields to singular strings
     *
     * @param User $object         User object
     * @param string $array_del    separator for complex data fields
     *
     * @return array
     */
    public function toFlatArray($object, $array_del = ",")
    {
        $secondary_delimiter = "; ";

        $raw = $this->extract($object);
        $export = [];
        foreach ($raw as $key => $value) {
            if (empty($value)) {
                continue;
            }
            if ($key == "data") {
                $line = "";
                foreach ($value as $ele => $fragment) {
                    if (empty($fragment)) {
                        continue;
                    }
                    $line .= $ele . ": " . $fragment . $array_del;
                }
                $export[$key] = rtrim($line, $array_del);
            } elseif ($key == "permissions") {
                $line = "";
                foreach ($value as $right => $fragment) {
                    $line .= " $right ($fragment)" . $array_del;
                }
                $export[$key] = rtrim($line, $array_del);
            } elseif ($key == "orders") {
                $line = "";
                foreach ($value as $order) {
                    $label = $order->getLabel();
                    $label = preg_replace('/\s+/', ' ', $label);
                    $label = str_replace($this::FIND, $this::REPLACE, $label);
                    $line .= $label . $secondary_delimiter;
                }
                $export[$key] = rtrim($line, ";");
            } elseif ($key == "addresses") {
                $line = "";
                foreach ($value as $addr) {
                    if (empty($addr)) {
                        continue;
                    }
                    $line .= $addr->getId() . ": " . $addr->getLine1() . " " .
                        $addr->getLine2() . "  " . $addr->getZip() . " " .
                        $addr->getCity() . ", " . $addr->getCountry() .
                        $array_del . " ";
                }
                $export[$key] = rtrim($line, $array_del);
            } else {
                $export[$key] = $value;
            }
        }
        return $export;
    }

    /**
     * Extracts data and transforms objects to arrays
     *
     * @param User $object         User object
     *
     * @return array
     */
    public function toArray($object)
    {
        $raw = $this->extract($object);
        $export = [];
        foreach ($raw as $key => $value) {
            if (empty($value)) {
                continue;
            }
            if ($key == "addresses") {
                $export[$key] = [];
                foreach ($value as $addr) {
                    if (empty($addr)) {
                        continue;
                    }
                    $temp = [
                        'id' => $addr->getId(),
                        'line1' => $addr->getLine1(),
                        'line2' => $addr->getLine2(),
                        'zip' => $addr->getZip(),
                        'city' => $addr->getCity(),
                        'country' => $addr->getCountry(),
                    ];
                    foreach ($temp as $k => $v) {
                        if (empty($v)) {
                            unset($temp[$k]);
                        }
                    }
                    $export[$key][$addr->getId()] = $temp;
                }
            } elseif ($key == "orders") {
                $export[$key] = [];
                foreach ($value as $order) {
                    $label = $order->getLabel();
                    $label = preg_replace('/\s+/', ' ', $label);
                    $export[$key][] = str_replace($this::FIND, $this::REPLACE, $label);
                }
            } else {
                $export[$key] = $value;
            }
        }
        return $export;
    }
}
